// RelativeLayers 0.9.7
// 
// $Id: rl_dragdrop.js 1.7 04/02/02 21:22:32+01:00 gbevin@willow.uwyn.office $
// 
// Library for creating dynamic designs that adapt themselves to the user's
// browser environment.
// 
// Copyright (C) 1998-2004, Geert Bevin
// Distributed under the terms of the GNU Lesser General Public, v2.1 or later
// 
// gbevin[remove] at uwyn dot com
// http://www.uwyn.com/projects/relativelayers

// define the draggable identifiers
var VERTDRAGGABLE = 3;
var HORIZDRAGGABLE = 2;
var DRAGGABLE = 1;
var NONDRAGGABLE = 0;

// define the drag limits identifiers
var NOLIMITS = 0;
var AVAILABLELIMITS = 1;

// variable containing the layer that's being dragged
var glb_draggingLayer = false;

// ARGUMENTS
// =========
// par_draggable : determines if a layer is draggable or not by setting it to VERTDRAGGABLE, HORIZDRAGGABLE, DRAGGABLE or NONDRAGGABLE
RelativeLayer.prototype.setDraggable = function(par_draggable)
{
	this.mbr_draggable = par_draggable;
	if (!this.mbr_dragLimit)
	{
		this.mbr_dragLimit = NOLIMITS;
	}
	if (!this.mbr_autoraise)
	{
		this.mbr_autoraise = false;
	}
	this.mbr_dragActive = false;

	if (this.mbr_draggable != NONDRAGGABLE)
	{
		this.setDragEvents();
	}
	else
	{
		this.removeDragEvents();
	}
};

// ARGUMENTS
// =========
// par_draglimit : determines the limits of the area into which the layer can dragged NOLIMITS or AVAILABLELIMITS
RelativeLayer.prototype.setDragLimit = function(par_draglimit)
{
	this.mbr_dragLimit = par_draglimit;
};

// ARGUMENTS
// =========
// par_autoraise : determine if the layer has to be moved automatically in front of others or not
RelativeLayer.prototype.setDragAutoRaise = function(par_autoraise)
{
	this.mbr_autoraise = par_autoraise;
};

RelativeLayer.prototype.addDropTarget = function(par_target)
{
	if (!this.mbr_droptargets)
	{
		this.mbr_droptargets = new Array();
	}

	if (!this.mbr_droptargets.contains(par_target))
	{
		this.mbr_droptargets.push(par_target);
	}
};

RelativeLayer.prototype.removeDropTarget = function(par_target)
{
	var var_result = false;

	if (this.mbr_droptargets)
	{
		var_result = this.mbr_droptargets.remove(par_target);
		if (this.mbr_droptargets.length == 0)
		{
			this.mbr_droptargets = null;
		}
	}

	return var_result;
};

RelativeLayer.prototype.setDragEvents = function()
{
	if (this.mbr_draggable != NONDRAGGABLE)
	{
		this.addEventListener("mousedown", this.dragMouseDown);
	}
};

RelativeLayer.prototype.removeDragEvents = function()
{
	this.removeEventListener("mousedown", this.dragMouseDown);
};

RelativeLayer.prototype.dragMouseDown = function(par_event)
{
	// filter out the right mousebuttons and unsupported browsers
	if (kq || par_event.which == 1)
	{
		// check if the layer is set as draggable and if no drag is currently active
		if (par_event.target &&
			par_event.target.mbr_draggable &&
			!par_event.target.mbr_dragActive)
		{
			var var_x = par_event.pageX;
			var var_y = par_event.pageY;

			// set the dragging begin position
			par_event.target.mbr_dragStartX = var_x;
			par_event.target.mbr_dragStartY = var_y;
			// set the offset from the absolute center position
			par_event.target.mbr_dragAbsoluteCenterX = par_event.target.getAbsoluteCenterX();
			par_event.target.mbr_dragAbsoluteCenterY = par_event.target.getAbsoluteCenterY();
			// set the offset from the visible area center
			par_event.target.mbr_dragOffsetX = var_x - par_event.target.mbr_dragAbsoluteCenterX;
			par_event.target.mbr_dragOffsetY = var_y - par_event.target.mbr_dragAbsoluteCenterY;
			// calculate the required dimensions
			par_event.target.mbr_calculatedVisibleWidth = par_event.target.calculateVisibleWidth();
			par_event.target.mbr_calculatedParentWidth = par_event.target.getParentWidth();
			par_event.target.mbr_calculatedVisibleHeight = par_event.target.calculateVisibleHeight();
			par_event.target.mbr_calculatedParentHeight = par_event.target.getParentHeight();
			par_event.target.mbr_calculatedAvailableHorizOffset = par_event.target.calculateAvailableHorizOffset(par_event.target.mbr_calculatedVisibleWidth, par_event.target.calculateAvailableWidth(par_event.target.mbr_calculatedVisibleWidth));
			par_event.target.mbr_calculatedAvailableVertOffset = par_event.target.calculateAvailableVertOffset(par_event.target.mbr_calculatedVisibleHeight, par_event.target.calculateAvailableHeight(par_event.target.mbr_calculatedVisibleHeight));

			// apply to autoraise settings
			if (par_event.target.mbr_autoraise == true)
			{
				par_event.target.raise();
			}

			// activate the dragging
			glb_draggingLayer = par_event.target;
			glb_draggingLayer.mbr_lastdroptarget = null;
			glb_draggingLayer.mbr_dragActive = true;
			glb_draggingLayer.mbr_dependinglayers = glb_draggingLayer.getDependingLayers();
			if (glb_draggingLayer.mbr_dependinglayers.length == 0)
			{
				glb_draggingLayer.mbr_dependinglayers = null;
			}

			// notify the registered event listeners
			var var_event_dragstart = new Object();
			var_event_dragstart.type = "dragstart";
			var_event_dragstart.target = par_event.target;
			var_event_dragstart.pageX = par_event.pageX;
			var_event_dragstart.pageY = par_event.pageY;
			var_event_dragstart.layerX = par_event.layerX;
			var_event_dragstart.layerY = par_event.layerY;
			var_event_dragstart.target.fireEventObject(var_event_dragstart);

			// capture all mouse move and mouse up events
			if (op || kq)
			{
				glb_draggingLayer.mbr_windowOnmousemove = document.onmousemove;
				glb_draggingLayer.mbr_windowOnmouseup = document.onmouseup;
				document.onmousemove = dragMouseMove;
				document.onmouseup = dragMouseUp;
			}
			else if (mz)
			{
				glb_draggingLayer.mbr_windowOnmousemove = window.onmousemove;
				glb_draggingLayer.mbr_windowOnmouseup = window.onmouseup;
				window.onmousemove = dragMouseMove;
				window.onmouseup = dragMouseUp;
				glb_draggingLayer.mbr_documentOnmousedown = document.onmousedown;
				document.onmousedown = cancelEvent;
			}
			else if (ns)
			{
				glb_draggingLayer.mbr_documentOnmousemove = document.onmousemove;
				glb_draggingLayer.mbr_documentOnmouseup = document.onmouseup;
				glb_draggingLayer.mbr_documentOnmousedown = document.onmousedown;
				document.onmousemove = dragMouseMove;
				document.onmouseup = dragMouseUp;
				document.onmousedown = cancelEvent;
				document.captureEvents(Event.MOUSEDOWN | Event.MOUSEMOVE | Event.MOUSEUP);
			}
			else if (ie)
			{
				glb_draggingLayer.mbr_windowOnmousemove = document.onmousemove;
				glb_draggingLayer.mbr_windowOnmouseup = document.onmouseup;
				document.onmousemove = dragMouseMove;
				document.onmouseup = dragMouseUp;
				glb_draggingLayer.mbr_documentOnselectstart = document.onselectstart;
				document.onselectstart = cancelEvent;
			}
		}
	}
};

function cancelEvent(par_event)
{
	return false;
};

function getPossibleDropTarget(par_glb_draggingLayer)
{
	var var_highest_target = null;

	if (par_glb_draggingLayer.mbr_droptargets)
	{
		var var_index;
		var var_hit_targets = new Array();
		for (var_index = 0; var_index < par_glb_draggingLayer.mbr_droptargets.length; var_index++)
		{
			if (par_glb_draggingLayer.intersects(par_glb_draggingLayer.mbr_droptargets[var_index]))
			{
				var_hit_targets.push(par_glb_draggingLayer.mbr_droptargets[var_index]);
			}
		}
		if (var_hit_targets.length > 0)
		{
			var_highest_target = var_hit_targets[0];
			for (var_index = 1; var_index < var_hit_targets.length; var_index++)
			{
				if (var_hit_targets[var_index].getZIndex() > var_highest_target.getZIndex())
				{
					var_highest_target = var_hit_targets[var_index];
				}
			}
		}
	}

	return var_highest_target;
};

function dragMouseMove(par_event)
{
	var var_event = normalizeEvent(par_event, this);

	if (var_event &&
		glb_draggingLayer &&
		glb_draggingLayer.mbr_dragActive)
	{
		var var_x = var_event.pageX;
		var var_y = var_event.pageY;

		// get the correct offset according to the visible area center
		if (glb_draggingLayer.mbr_draggable == DRAGGABLE)
		{
			var_x = (var_x - glb_draggingLayer.mbr_dragOffsetX) - glb_draggingLayer.mbr_dragAbsoluteCenterX;
			var_y = (var_y - glb_draggingLayer.mbr_dragOffsetY) - glb_draggingLayer.mbr_dragAbsoluteCenterY;
		}

		else if (glb_draggingLayer.mbr_draggable == VERTDRAGGABLE)
		{
			var_x = 0;
			var_y = (var_y - glb_draggingLayer.mbr_dragOffsetY) - glb_draggingLayer.mbr_dragAbsoluteCenterY;
		}
		else if (glb_draggingLayer.mbr_draggable == HORIZDRAGGABLE)
		{
			var_x = (var_x - glb_draggingLayer.mbr_dragOffsetX) - glb_draggingLayer.mbr_dragAbsoluteCenterX;
			var_y = 0;
		}

		// constrain the movement of the draggable layer to the available surface
		if (glb_draggingLayer.mbr_dragLimit == AVAILABLELIMITS)
		{
			if ((glb_draggingLayer.getAbsoluteX() - glb_draggingLayer.mbr_calculatedAvailableHorizOffset + var_x) < 0)
			{
				var_x = -(glb_draggingLayer.getAbsoluteX() - glb_draggingLayer.mbr_calculatedAvailableHorizOffset);
			}
			else if ((glb_draggingLayer.getAbsoluteX() + glb_draggingLayer.mbr_calculatedVisibleWidth - glb_draggingLayer.mbr_calculatedAvailableHorizOffset + var_x) > glb_draggingLayer.mbr_calculatedParentWidth)
			{
				var_x = glb_draggingLayer.mbr_calculatedParentWidth - glb_draggingLayer.mbr_calculatedVisibleWidth - glb_draggingLayer.getAbsoluteX() + glb_draggingLayer.mbr_calculatedAvailableHorizOffset;
			}

			if ((glb_draggingLayer.getAbsoluteY() - glb_draggingLayer.mbr_calculatedAvailableVertOffset + var_y) < 0)
			{
				var_y = -(glb_draggingLayer.getAbsoluteY() - glb_draggingLayer.mbr_calculatedAvailableVertOffset);
			}
			else if ((glb_draggingLayer.getAbsoluteY() + glb_draggingLayer.mbr_calculatedVisibleHeight - glb_draggingLayer.mbr_calculatedAvailableVertOffset + var_y) > glb_draggingLayer.mbr_calculatedParentHeight)
			{
				var_y = glb_draggingLayer.mbr_calculatedParentHeight - glb_draggingLayer.mbr_calculatedVisibleHeight - glb_draggingLayer.getAbsoluteY() + glb_draggingLayer.mbr_calculatedAvailableVertOffset;
			}
		}

		if (var_x != 0 || var_y != 0)
		{
			// move the visible layer area to the new position
			glb_draggingLayer.mbr_dragAbsoluteCenterX += var_x;
			glb_draggingLayer.mbr_dragAbsoluteCenterY += var_y;
			glb_draggingLayer.absoluteMoveBy(var_x, var_y);

			// move the depending layers, but not the children
			if (glb_draggingLayer.mbr_dependinglayers)
			{
				var var_index;
				for (var_index = 0; var_index < glb_draggingLayer.mbr_dependinglayers.length; var_index++)
				{
					if (glb_draggingLayer.mbr_dependinglayers[var_index].getParentLayer() != glb_draggingLayer)
					{
						glb_draggingLayer.mbr_dependinglayers[var_index].absoluteMoveBy(var_x, var_y);
					}
				}
			}

			// notify the registered event listeners
			var var_event_dragmoved = new Object();
			var_event_dragmoved.type = "dragmoved";
			var_event_dragmoved.target = glb_draggingLayer;
			var_event_dragmoved.pageX = var_event.pageX;
			var_event_dragmoved.pageY = var_event.pageY;
			var_event_dragmoved.layerX = var_event.pageX-glb_draggingLayer.getAbsolutePageX();
			var_event_dragmoved.layerY = var_event.pageY-glb_draggingLayer.getAbsolutePageY();
			var_event_dragmoved.deltaX = var_x;
			var_event_dragmoved.deltaY = var_y;
			glb_draggingLayer.fireEventObject(var_event_dragmoved);
		}

		var var_droptarget = getPossibleDropTarget(glb_draggingLayer);
		if (glb_draggingLayer.mbr_lastdroptarget != var_droptarget)
		{
			if (glb_draggingLayer.mbr_lastdroptarget)
			{
				var var_event_dragleave = new Object();
				var_event_dragleave.type = "dragleave";
				var_event_dragleave.target = glb_draggingLayer.mbr_lastdroptarget;
				var_event_dragleave.pageX = var_event.pageX;
				var_event_dragleave.pageY = var_event.pageY;
				var_event_dragleave.source = glb_draggingLayer;
				glb_draggingLayer.mbr_lastdroptarget.fireEventObject(var_event_dragleave);
			}

			glb_draggingLayer.mbr_lastdroptarget = var_droptarget;

			if (var_droptarget)
			{
				var var_event_dragenter = new Object();
				var_event_dragenter.type = "dragenter";
				var_event_dragenter.target = var_droptarget;
				var_event_dragenter.pageX = var_event.pageX;
				var_event_dragenter.pageY = var_event.pageY;
				var_event_dragenter.source = glb_draggingLayer;
				var_droptarget.fireEventObject(var_event_dragenter);
			}
		}
	}

	return false;
};

function dragMouseUp(par_event)
{
	var var_event = normalizeEvent(par_event, this);

	// check if the dragging is active
	if (glb_draggingLayer &&
		glb_draggingLayer.mbr_dragActive)
	{
		var var_x = var_event.pageX;
		var var_y = var_event.pageY;

		// get the correct offset according to the visible area center
		if (glb_draggingLayer.mbr_draggable == DRAGGABLE)
		{
			var_x = var_x - glb_draggingLayer.mbr_dragStartX;
			var_y = var_y - glb_draggingLayer.mbr_dragStartY;
		}
		else if (glb_draggingLayer.mbr_draggable == VERTDRAGGABLE)
		{
			var_x = 0;
			var_y = var_y - glb_draggingLayer.mbr_dragStartY;
		}
		else if (glb_draggingLayer.mbr_draggable == HORIZDRAGGABLE)
		{
			var_x = var_x - glb_draggingLayer.mbr_dragStartX;
			var_y = 0;
		}

		// constrain the movement of the draggable layer to the available surface
		if (glb_draggingLayer.mbr_dragLimit == AVAILABLELIMITS)
		{
			if ((glb_draggingLayer.calculateVisibleHorizOffset() + var_x) < 0)
			{
				var_x = -glb_draggingLayer.calculateVisibleHorizOffset();
			}
			else if ((glb_draggingLayer.calculateVisibleHorizOffset() + glb_draggingLayer.mbr_calculatedVisibleWidth + var_x) > glb_draggingLayer.mbr_calculatedParentWidth)
			{
				var_x = glb_draggingLayer.mbr_calculatedParentWidth - glb_draggingLayer.mbr_calculatedVisibleWidth - glb_draggingLayer.calculateVisibleHorizOffset();
			}

			if ((glb_draggingLayer.calculateVisibleVertOffset() + var_y) < 0)
			{
				var_y = -glb_draggingLayer.calculateVisibleVertOffset();
			}
			else if ((glb_draggingLayer.calculateVisibleVertOffset() + glb_draggingLayer.mbr_calculatedVisibleHeight + var_y) > glb_draggingLayer.mbr_calculatedParentHeight)
			{
				var_y = glb_draggingLayer.mbr_calculatedParentHeight - glb_draggingLayer.mbr_calculatedVisibleHeight - glb_draggingLayer.calculateVisibleVertOffset();
			}
		}

		// perform the modifications
		glb_draggingLayer.visibleMoveBySetOffset(var_x, var_y);

		// update the document dimensions
		updateDocumentDimensions(glb_draggingLayer.mbr_id);

		// notify the registered event listeners
		var var_event_dragend = new Object();
		var_event_dragend.type = "dragend";
		var_event_dragend.target = glb_draggingLayer;
		var_event_dragend.pageX = var_event.pageX;
		var_event_dragend.pageY = var_event.pageY;
		var_event_dragend.layerX = var_event.pageX-glb_draggingLayer.getAbsolutePageX();
		var_event_dragend.layerY = var_event.pageY-glb_draggingLayer.getAbsolutePageY();
		glb_draggingLayer.fireEventObject(var_event_dragend);

		if (glb_draggingLayer.mbr_lastdroptarget)
		{
			var var_event_dragdrop = new Object();
			var_event_dragdrop.type = "dragdrop";
			var_event_dragdrop.target = glb_draggingLayer.mbr_lastdroptarget;
			var_event_dragdrop.pageX = var_event.pageX;
			var_event_dragdrop.pageY = var_event.pageY;
			var_event_dragdrop.source = glb_draggingLayer;
			glb_draggingLayer.mbr_lastdroptarget.fireEventObject(var_event_dragdrop);
		}

		// disable the dragging
		glb_draggingLayer.mbr_dependinglayers = null;
		glb_draggingLayer.mbr_lastdroptarget = null;
		glb_draggingLayer.mbr_dragActive = false;
		glb_draggingLayer = false;

		// restore the previous window mouse move and up events
		if (op)
		{
			if (glb_draggingLayer.mbr_windowOnmousemove)
			{
				document.onmousemove = glb_draggingLayer.mbr_windowOnmousemove;
			}
			else
			{
				document.onmousemove = null;
			}
			if (glb_draggingLayer.mbr_windowOnmouseup)
			{
				document.onmouseup = glb_draggingLayer.mbr_windowOnmouseup;
			}
			else
			{
				document.onmouseup = null;
			}
		}
		else if (mz || kq)
		{
			if (glb_draggingLayer.mbr_windowOnmousemove)
			{
				window.onmousemove = glb_draggingLayer.mbr_windowOnmousemove;
			}
			else
			{
				window.onmousemove = null;
			}
			if (glb_draggingLayer.mbr_windowOnmouseup)
			{
				window.onmouseup = glb_draggingLayer.mbr_windowOnmouseup;
			}
			else
			{
				window.onmouseup = null;
			}
			if (glb_draggingLayer.mbr_documentOnmousedown)
			{
				document.onmousedown = glb_draggingLayer.mbr_documentOnmousedown;
			}
			else
			{
				document.onmousedown = null;
			}
		}
		else if (ns)
		{
			if (glb_draggingLayer.mbr_documentOnmousemove)
			{
				document.onmousemove = glb_draggingLayer.mbr_documentOnmousemove;
			}
			else
			{
				document.onmousemove = null;
			}
			if (glb_draggingLayer.mbr_documentOnmouseup)
			{
				document.onmouseup = glb_draggingLayer.mbr_documentOnmouseup;
			}
			else
			{
				document.onmouseup = null;
			}
			if (glb_draggingLayer.mbr_documentOnmousedown)
			{
				document.onmousedown = glb_draggingLayer.mbr_documentOnmousedown;
			}
			else
			{
				document.onmousedown = null;
			}
			// this is not good, it should restore the state of the capture events before the drag
			document.releaseEvents(Event.MOUSEMOVE | Event.MOUSEUP | Event.MOUSEDOWN);
		}
		else if (ie)
		{
			if (glb_draggingLayer.mbr_windowOnmousemove)
			{
				document.onmousemove = glb_draggingLayer.mbr_windowOnmousemove;
			}
			else
			{
				document.onmousemove = null;
			}
			if (glb_draggingLayer.mbr_windowOnmouseup)
			{
				document.onmouseup = glb_draggingLayer.mbr_windowOnmouseup;
			}
			else
			{
				document.onmouseup = null;
			}
			if (glb_draggingLayer.mbr_documentOnselectstart)
			{
				document.onselectstart = glb_draggingLayer.mbr_documentOnselectstart;
			}
			else
			{
				document.onselectstart = null;
			}
		}

		glb_draggingLayer.mbr_windowOnmousemove = null;
		glb_draggingLayer.mbr_windowOnmouseup = null;

		// should the document size be changed due to for example scrollbars additions,
		// consider it as a windows resize
		if (checkResize())
		{
			processResizeHooks();
			storeWindowDimensions();
		}
	}
};

