// RelativeLayers 0.9.7
// 
// $Id: rl_window.js 1.7 04/02/02 21:22:32+01:00 gbevin@willow.uwyn.office $
// 
// Library for creating dynamic designs that adapt themselves to the user's
// browser environment.
// 
// Copyright (C) 1998-2004, Geert Bevin
// Distributed under the terms of the GNU Lesser General Public, v2.1 or later
// 
// gbevin[remove] at uwyn dot com
// http://www.uwyn.com/projects/relativelayers

window.onload = initHandler;
window.onresize = resizeHandler;

var glb_document_loaded = false;
var glb_document_resized = false;

var glb_previous_window_width = -1;
var glb_previous_window_height = -1;

var glb_processing_hooks = null;
var glb_load_hooks = new Array();
var glb_current_loadhook = -1;
var glb_resize_hooks = new Array();
var glb_current_resizehook = -1;

function initHandler()
{
	glb_document_loaded = true;
	if (ns)
	{
		window.captureEvents(Event.RESIZE);
	}

	storeWindowDimensions();

	processLoadHooks();
};

function resizeHandler()
{
	// can't seem to make this work for opera
	if (checkResize() && 
		glb_document_resized)
	{
		glb_document_resized = false;
		processResizeHooks();
	}
};

function nextHook()
{
	if (glb_processing_hooks == glb_load_hooks)
	{
		processLoadHooks();
	}
	else if (glb_processing_hooks == glb_resize_hooks)
	{
		processResizeHooks();
	}
};

function addLoadHook(par_hook, par_position)
{
	if (arguments.length < 2)
	{
		par_position = glb_load_hooks.length;
	}

	if (glb_load_hooks.length-1 >= par_position &&
		glb_load_hooks[par_position])
	{
		var var_index = par_position;
		while (var_index < glb_load_hooks.length &&
			   glb_load_hooks[var_index])
		{
			var_index++;
		}
		par_position = var_index;
	}

	glb_load_hooks[par_position] = par_hook;
};

function processLoadHooks()
{
	var var_index;

	glb_processing_hooks = glb_load_hooks;

	for (var_index = glb_current_loadhook+1; var_index < glb_load_hooks.length; var_index++)
	{
		if (glb_load_hooks[var_index])
		{
			glb_current_loadhook = var_index;
			if (glb_load_hooks[var_index]() == false)
			{
				break;
			}
		}
	}

	if (var_index >= glb_load_hooks.length)
	{
		glb_document_resized = true;
		glb_processing_hooks = null;
		glb_current_loadhook = -1;
	}
};

function addResizeHook(par_hook, par_position)
{
	if (arguments.length < 2)
	{
		par_position = glb_resize_hooks.length;
	}

	if (glb_resize_hooks.length-1 >= par_position &&
		glb_resize_hooks[par_position])
	{
		var var_index = par_position;
		while (var_index < glb_resize_hooks.length &&
			   glb_resize_hooks[var_index])
		{
			var_index++;
		}
		par_position = var_index;
	}

	glb_resize_hooks[par_position] = par_hook;
};

function processResizeHooks()
{
	var var_index;

	glb_processing_hooks = glb_resize_hooks;

	for (var_index = glb_current_resizehook+1; var_index < glb_resize_hooks.length; var_index++)
	{
		if (glb_resize_hooks[var_index])
		{
			glb_current_resizehook = var_index;
			if (glb_resize_hooks[var_index]() == false)
			{
				break;
			}
		}
	}

	if (var_index >= glb_resize_hooks.length)
	{
		glb_document_resized = true;
		glb_processing_hooks = null;
		glb_current_resizehook = -1;
	}
};

// check if the window was really resized, or if it's a recursive resize
function checkResize()
{
	var var_isresized = false;

	if ((getWindowWidth() != glb_previous_window_width) ||
		(getWindowHeight() != glb_previous_window_height))
	{
		var_isresized = true;
	}

	return var_isresized;
};

// store the window dimension, to ensure that no recursive resive events occur
function storeWindowDimensions()
{
	glb_previous_window_width = getWindowWidth();
	glb_previous_window_height = getWindowHeight();
};

// functions to get the window dimensions
function getWindowWidth()
{
	var var_windowwidth = 0;

	if (op || ns || kq)
	{
		var_windowwidth = window.innerWidth;
	}
	else if (mz)
	{
		if (window.SIZER_LAYER &&
			window.SIZER_LAYER.offsetWidth < window.innerWidth)
		{
			var_windowwidth = window.SIZER_LAYER.offsetWidth;
		}
		else
		{
			var_windowwidth = window.innerWidth;
		}
	}
	else if (ie)
	{
		var_windowwidth = document.body.clientWidth;
	}

	return var_windowwidth;
};

function getWindowHeight()
{
	var var_windowheight = 0;

	if (op || ns || kq)
	{
		var_windowheight = window.innerHeight;
	}
	else if (mz)
	{
		if (window.SIZER_LAYER &&
			window.SIZER_LAYER.offsetHeight < window.innerHeight)
		{
			var_windowheight = window.SIZER_LAYER.offsetHeight;
		}
		else
		{
			var_windowheight = window.innerHeight;
		}
	}
	else if (ie)
	{
		var_windowheight = document.body.clientHeight;
	}

	return var_windowheight;
};

function updateDocumentDimensions(par_layer)
{
	if (par_layer)
	{
		// update the document's dimensions for netscape since it doesn't do so interactively
		if (ns)
		{
			// only for the 1st level layers, not for nested ones
			if (!par_layer.parentLayer ||
				!par_layer.parentLayer.id)
			{
				var top_layer;

				var right_edge;
				var bottom_edge;

				var document_width = 0;
				var document_height = 0;

				for (var i = 0; i < document.layers.length; i++)
				{
					top_layer = document.layers[i];
					right_edge = Math.floor(top_layer.left)+Math.floor(top_layer.clip.width);
					bottom_edge = Math.floor(top_layer.top)+Math.floor(top_layer.clip.height);

					if (document_width < right_edge)
					{
						document_width = right_edge;
					}

					if (document_height < bottom_edge)
					{
						document_height = bottom_edge;
					}
				}

				// ensure that the visible area continues to correspond to the scrollbars
				if (window.pageXOffset > 0 &&
					document_width < document.width)
				{
					var scroll_amount = document.width-document_width;
					if (scroll_amount > window.pageXOffset)
					{
						scroll_amount = window.pageXOffset; 
					}
					window.scrollBy(-1*scroll_amount, 0);
				}
				if (window.pageYOffset > 0 &&
					document_height < document.height)
				{
					var scroll_amount = document.height-document_height;
					if (scroll_amount > window.pageYOffset)
					{
						scroll_amount = window.pageYOffset; 
					}
					window.scrollBy(0, -1*scroll_amount);
				}

				// update the document dimensions
				document.width = document_width;
				document.height = document_height;
			}
		}
	}
};
